--[[
************************************************************************
Project				: Broker_HitCrit
Author				: zhinjio
Project Revision	: r5
Project Date		: 20081102084324

File				: Broker_HitCrit.lua
Commit Author		: zhinjio
Commit Revision		: 5
Commit Date			: 20081102084324
************************************************************************
Description	:
	Main addon file. 

TODO		:
	Figure out sound notify
	Figure out notify area notify

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

-- For now, we need a hack in here to make sure ace3 loads before I do.
-- All hail disembeded libraries and their swift return....
LoadAddOn("Ace3")

HitCrit_VERSION = 0.1
-- SKG_DEBUG = true
HitCrit_DEBUG = true

local HitCrit = LibStub( "AceAddon-3.0" ):NewAddon( "HitCrit", "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( "HitCrit" )
_G["HitCrit"] = HitCrit

local tooltip
local LTT = LibStub:GetLibrary( "LibTooltip-1.0" )

local db

function HitCrit_Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\Broker_HitCrit\\icon:" .. fh .. "\124t"
	if not ( msg == nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " HitCrit: " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function HitCrit_Debug( msg )
	if not ( msg == nil ) then
		if ( HitCrit_DEBUG ) then
			HitCrit_Print( "DEBUG: " .. msg )
		end
	end
end

function HitCrit_Red( text )
	return "|cffff0000" .. text .. "|r"
end
function HitCrit_Yellow( text )
	return "|cffffff00" .. text .. "|r"
end
function HitCrit_Green( text )
	return "|cff00ff00" .. text .. "|r"
end
function HitCrit_White( text )
	return "|cffffffff" .. text .. "|r"
end
function HitCrit_hit( text)
	return "|cff00ffff" .. text .. "|r"
end
function HitCrit_crit( text)
	return "|cffff8080" .. text .. "|r"
end

function HitCrit:AlertNew( crit, heal, target, spellName, amount )
	local hType, hStr = L["Hit"], L["hit"]
	if ( heal == 1 ) then
		hStr = L["healed"]
		if ( crit == 1 ) then
			hType = L["Critical Heal"]
		else
			hType = L["Heal"]
		end
	else
		if ( crit == 1 ) then
			hType = L["Critical Hit"]
		end
	end
	if ( db.options.alerts.chatframe == true ) then
		HitCrit_Print( string.format( L["New Record %s! %s %s %s for %d"], hType, spellName, hStr, target, amount ) )
	end
	if ( db.options.alerts.notify == true ) then
		-- Still need to figure this out
	end
	if ( db.options.alerts.sounds == true ) then
		-- Still need to figure this out
	end
end

function HitCrit:COMBAT_LOG_EVENT_UNFILTERED( _, _, eType, sGUID, sName, sFlags, dGUID, dName, dFlags, ... )
-- Parameters:
--	event		COMBAT_LOG_EVENT_UNFILTERED
--	timestamp
--	eType		event type (for example, SPELL_DAMAGE)
--	sGUID		source GUID
--	sName		source Name
--	sFlags		source Flags
--	dGUID		destination GUID
--	dName		destination Name
--	dFlags		destination Flags
--
--		The remaining parameters are dependent on eType
--
--		For RANGE_, SPELL_, SPELL_AURA_ and SPELL_PERIODIC_ prefixes, you have the following
--			spellId
--			spellName
--			spellSchool
--
--		Then, for _DAMAGE suffixes:
--			amount
--			overkill
--			school
--			resisted
--			blocked
--			absorbed
--			critical (1 or nil)
--			glancing (1 or nil)
--			crushing (1 or nil)
--
--		Then, for _HEAL suffixes:
--			amount
--			overhealing
--			critical
--
	local effect, id, hit, crit, sch, heal = "", 0, 0, 0, 0, 0
	if ( sGUID == HitCrit.GUID ) then
		-- First, check for damage
		if ( db.options.tracking.damage ) then
			if ( eType == "SWING_DAMAGE" ) then
				local amount, _, school, resisted, blocked, absorbed, critical, glance, crush = select( 1, ... )
				id = 0; effect = L["Melee"]; sch = school
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "RANGE_DAMAGE" ) then
				local spellID, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glance, crush = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "SPELL_DAMAGE" ) then
				local spellID, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glance, crush = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "SPELL_AURA_DAMAGE" ) then
				-- does this event exist?
				local spellID, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glance, crush = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "SPELL_PERIODIC_DAMAGE" ) then
				local spellID, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, crit, glance, crush = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			end
		end
		if ( db.options.tracking.healing ) then
			if ( eType == "SWING_HEAL" ) then
				-- does this event exist?
				local amount, overheal, crit = select( 1, ... )
				id = 0; effect = L["Melee"]; sch = 1; heal = 1
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "RANGE_HEAL" ) then
				-- does this event exist?
				local spellID, spellName, spellSchool, amount, overheal, critical = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool; heal = 1
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "SPELL_HEAL" ) then
				local spellID, spellName, spellSchool, amount, overheal, critical = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool; heal = 1
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			elseif ( eType == "SPELL_AURA_HEAL" ) then
				-- does this event exist?
				local spellID, spellName, spellSchool, amount, overheal, critical = select( 1, ... )
				id = spellID; effect = spellName; sch = spellSchool; heal = 1
				if ( critical == 1 ) then hit = 0; crit = amount else hit = amount; crit = 0 end
			end
		end
	elseif ( ( db.options.tracking.againstMe == true ) and ( dGUID == HitCrit.GUID ) ) then
	end
	if ( effect ~= "" ) then
		if ( db.values[id] ) then
			-- this value already exists. Compare to current, and replace if needed
			if ( crit ~= 0 ) then
				if ( crit > db.values[id].crit ) then
					HitCrit:AlertNew( 1, heal, dName, effect, crit )
					db.values[id].crit = crit
				end
			else
				if ( hit > db.values[id].hit ) then
					HitCrit:AlertNew( 0, heal, dName, effect, hit )
					db.values[id].hit = hit
				end
			end
		else
			-- Otherwise, we've got a new previously untracked value. Record all the info
			db.values[id] = {}
			db.values[id].hit = hit
			db.values[id].crit = crit
			db.values[id].effect = effect
			db.values[id].target = dName
			db.values[id].school = sch
			if ( crit ~= 0 ) then
				HitCrit:AlertNew( 1, heal, dName, effect, crit )
			else
				HitCrit:AlertNew( 0, heal, dName, effect, hit )
			end
			if ( heal == 1 ) then
				db.values[id].dmg = 0
			else
				db.values[id].dmg = 1
			end
		end
	end
end

local LDB = LibStub( "LibDataBroker-1.1" )
local launcher = LDB:NewDataObject( "HitCrit", {
	type = "data source",
	text = "HitCrit",
	label = "Broker_HitCrit",
	icon = "Interface\\Addons\\Broker_HitCrit\\icon",
	OnEnter = function ( self )
		tooltip:Clear()
		local myFont = CreateFont("MyFont")
		if ( db.options.display.fontsize == "small" ) then
			myFont:CopyFontObject(SystemFont_Tiny)
			tooltip:SetFont(myFont)
			myFont:CopyFontObject(SystemFont_Med1)
			tooltip:SetHeaderFont(myFont)
		else
			myFont:CopyFontObject(SystemFont_Small)
			tooltip:SetFont(myFont)
			myFont:CopyFontObject(SystemFont_Med2)
			tooltip:SetHeaderFont(myFont)
		end
		tooltip:AddHeader( HitCrit_White( L["Broker_HitCrit"] ) )
		if ( db.options.display.type ) then
			-- sort by damage school
		else
			tooltip:AddLine( HitCrit_Red( "  " .. L["Damage"] ) )
			tooltip:AddLine( HitCrit_White( "    " .. L["Effect"] ), HitCrit_White( " " ), HitCrit_White( L["Hit"] ),
				HitCrit_White( " " ), HitCrit_White( L["Crit"] ) )
			-- loop through damage
			for k, v in pairs( db.values ) do
				if ( v.dmg == 1 ) then
					tooltip:AddLine( HitCrit_Yellow( "    " .. v.effect ), HitCrit_White( " " ), HitCrit_hit( v.hit ),
						HitCrit_White( " " ), HitCrit_crit( v.crit ) )
				end
			end
			tooltip:AddLine( " " )
			tooltip:AddLine( HitCrit_Green( "  " .. L["Healing"] ) )
			tooltip:AddLine( HitCrit_White( "    " .. L["Effect"] ), HitCrit_White( " " ), HitCrit_White( L["Hit"] ),
				HitCrit_White( " " ), HitCrit_White( L["Crit"] ) )
			-- loop through healing
			for k, v in pairs( db.values ) do
				if ( v.dmg == 0 ) then
					tooltip:AddLine( HitCrit_Yellow( "    " .. v.effect ), HitCrit_White( " " ), HitCrit_hit( v.hit ),
						HitCrit_White( " " ), HitCrit_crit( v.crit ) )
				end
			end
		end
		tooltip:SmartAnchorTo( self )
		tooltip:Show()
	end,
	OnLeave = function ( )
		tooltip:Hide()
	end,
} )

local defOpts = {
}

-- on init
function HitCrit:OnInitialize()
    -- Called when the addon is loaded
	HitCrit.me = UnitName( "player" )
	HitCrit.GUID = UnitGUID( "player" )
	HitCrit.level = UnitLevel( "player" )

	self.db = LibStub( "AceDB-3.0" ):New( "HitCrit_DB", nil, "Default" )
	-- do we have any data yet for this toon? If not, init its table
	if not ( self.db.factionrealm[HitCrit.me] ) then
		self.db.factionrealm[HitCrit.me] = {
			options = {
				tracking = {
					pvp = false,
					againstMe = false,
					damage = true,
					healing = true,
					environmental = false,
				},
				display = {
					hit = true,
					crit = true,
					enemyName = false,
					type = false,
					fontsize = "small",
				},
				alerts = {
					sound = false,
					notify = false,
					chatframe = true,
				},
			},
		-- Values will be in the following format:
		--	SpellID = {
		--		effect = spellName,
		--		hit = value,
		--		crit = value,
		--		dmg = 1, (0 for heal)
		--		target = Name,
		--		school = schoolID,
		--	} -- In the case of normal white melee damage, SpellID will be 0
			values = {
			}
		}
	end
	db = self.db.factionrealm[HitCrit.me]

	HitCrit_Debug( "In OnInit" )
	-- Event listeners
    HitCrit:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	tooltip = LTT:Acquire( "HitCritTT", 5, "LEFT", "CENTER", "RIGHT", "CENTER", "RIGHT" ) -- or GameTooltip
end

--[[
************************************************************************
CHANGELOG:

Date : 10/29/08
	Initial version
************************************************************************
]]--